@@ -2,7 +2,7 @@ import copy
 from enum import IntEnum
 import threading
 from decimal import Decimal
-from typing import Optional, TYPE_CHECKING
+from typing import Optional, TYPE_CHECKING, Callable
 from functools import partial
 
 from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject, pyqtEnum
@@ -11,13 +11,15 @@ from electrum.logging import get_logger
 from electrum.i18n import _
 from electrum.bitcoin import DummyAddress
 from electrum.transaction import PartialTxOutput, PartialTransaction, Transaction, TxOutpoint
-from electrum.util import NotEnoughFunds, profiler, quantize_feerate, UserFacingException
+from electrum.util import NotEnoughFunds, profiler, quantize_feerate, UserFacingException, NoDynamicFeeEstimates
 from electrum.wallet import CannotBumpFee, CannotDoubleSpendTx, CannotCPFP, BumpFeeStrategy, sweep_preparations
 from electrum import keystore
 from electrum.plugin import run_hook
 from electrum.fee_policy import FeePolicy, FeeMethod
 from electrum.network import NetworkException
 
+from electrum.gui import messages
+
 from .qewallet import QEWallet
 from .qetypes import QEAmount
 from .util import QtEventListener, event_listener
@@ -33,12 +35,14 @@ class FeeSlider(QObject):
         FEERATE = 0
         ETA = 1
         MEMPOOL = 2
+        MANUAL = 3
 
         def to_fee_method(self) -> 'FeeMethod':
             return {
                 self.FEERATE: FeeMethod.FEERATE,
                 self.ETA: FeeMethod.ETA,
                 self.MEMPOOL: FeeMethod.MEMPOOL,
+                self.MANUAL: FeeMethod.FIXED
             }[self]
 
         @classmethod
@@ -47,6 +51,7 @@ class FeeSlider(QObject):
                 FeeMethod.FEERATE: cls.FEERATE,
                 FeeMethod.ETA: cls.ETA,
                 FeeMethod.MEMPOOL: cls.MEMPOOL,
+                FeeMethod.FIXED: cls.MANUAL
             }[fm]
 
     def __init__(self, parent=None):
@@ -55,7 +60,8 @@ class FeeSlider(QObject):
         self._wallet = None  # type: Optional[QEWallet]
         self._sliderSteps = 0
         self._sliderPos = 0
-        self._fee_policy = None
+        self._fee_method = None  # type: Optional[FeeSlider.FSMethod]
+        self._fee_policy = None  # type: Optional[FeePolicy]
         self._target = ''
         self._config = None  # type: Optional[SimpleConfig]
 
@@ -97,10 +103,9 @@ class FeeSlider(QObject):
 
     @method.setter
     def method(self, method: int):
-        fsmethod = self.FSMethod(method)
-        method = fsmethod.to_fee_method()
-        if self._fee_policy.method != method:
-            self._fee_policy.set_method(method)
+        if self._fee_method != FeeSlider.FSMethod(method):
+            self._fee_method = self.FSMethod(method)
+            self._fee_policy.set_method(self._fee_method.to_fee_method())
             self.update_slider()
             self.methodChanged.emit()
             self.save_config()
@@ -117,6 +122,8 @@ class FeeSlider(QObject):
             self.targetChanged.emit()
 
     def update_slider(self):
+        if self._fee_method == FeeSlider.FSMethod.MANUAL:
+            return
         self._sliderSteps = self._fee_policy.get_slider_max()
         self._sliderPos = self._fee_policy.get_slider_pos()
         self.sliderStepsChanged.emit()
@@ -127,16 +134,16 @@ class FeeSlider(QObject):
 
     def read_config(self):
         self._fee_policy = FeePolicy(self._config.FEE_POLICY)
+        self._fee_method = self.FSMethod.from_fee_method(self._fee_policy.method)
         self.update_slider()
         self.methodChanged.emit()
-        self.update_target()
         self.update()
 
     def save_config(self):
-        value = int(self._sliderPos)
-        self._fee_policy.set_value_from_slider_pos(value)
-        self._config.FEE_POLICY = self._fee_policy.get_descriptor()
-        self.update_target()
+        if self._fee_method != FeeSlider.FSMethod.MANUAL:
+            value = int(self._sliderPos)
+            self._fee_policy.set_value_from_slider_pos(value)
+            self._config.FEE_POLICY = self._fee_policy.get_descriptor()
         self.update()
 
     def update(self):
@@ -149,8 +156,11 @@ class TxFeeSlider(FeeSlider):
 
         self._fee = QEAmount()
         self._feeRate = ''
+        self._userFee = ''
+        self._userFeerate = ''
+        self._is_user_feerate_last = True
         self._rbf = False
-        self._tx = None
+        self._tx = None  # type: Optional[PartialTransaction]
         self._inputs = []
         self._outputs = []
         self._finalized_txid = ''
@@ -179,6 +189,50 @@ class TxFeeSlider(FeeSlider):
             self._feeRate = feeRate
             self.feeRateChanged.emit()
 
+    userFeeChanged = pyqtSignal()
+    @pyqtProperty(str, notify=userFeeChanged)
+    def userFee(self):
+        return self._userFee
+
+    @userFee.setter
+    def userFee(self, userFee):
+        if self._userFee != userFee:
+            self._logger.warn('userFee')
+            self._userFee = userFee
+            user_fee = int(userFee) if userFee else 0
+            self._fee_policy = FeePolicy(f'fixed:{user_fee}')
+            self.userFeeChanged.emit()
+            self.isUserFeerateLast = False
+            self.update()
+
+    userFeerateChanged = pyqtSignal()
+    @pyqtProperty(str, notify=userFeerateChanged)
+    def userFeerate(self):
+        return self._userFeerate
+
+    @userFeerate.setter
+    def userFeerate(self, userFeerate):
+        if self._userFeerate != userFeerate:
+            self._logger.warn('userFeerate')
+            self._userFeerate = userFeerate
+            as_decimal = Decimal(userFeerate) if userFeerate else 0
+            user_feerate = int(as_decimal * 1000)
+            self._fee_policy = FeePolicy(f'feerate:{user_feerate}')
+            self.userFeerateChanged.emit()
+            self.isUserFeerateLast = True
+            self.update()
+
+    isUserFeerateLastChanged = pyqtSignal()
+    @pyqtProperty(bool, notify=isUserFeerateLastChanged)
+    def isUserFeerateLast(self):
+        return self._is_user_feerate_last
+
+    @isUserFeerateLast.setter
+    def isUserFeerateLast(self, isUserFeerateLast):
+        if self._is_user_feerate_last != isUserFeerateLast:
+            self._is_user_feerate_last = isUserFeerateLast
+            self.isUserFeerateLastChanged.emit()
+
     rbfChanged = pyqtSignal()
     @pyqtProperty(bool, notify=rbfChanged)
     def rbf(self):
@@ -244,7 +298,7 @@ class TxFeeSlider(FeeSlider):
     def doUpdate(self):
         self.update()
 
-    def update_from_tx(self, tx):
+    def update_from_tx(self, tx: PartialTransaction):
         tx_size = tx.estimated_size()
         fee = tx.get_fee()
         feerate = Decimal(fee) / tx_size  # sat/byte
@@ -255,8 +309,19 @@ class TxFeeSlider(FeeSlider):
 
         self.update_inputs_from_tx(tx)
         self.update_outputs_from_tx(tx)
+        self.update_target()
+        self.update_manual_fields()
 
-    def update_inputs_from_tx(self, tx):
+    def update_manual_fields(self):
+        if self._fee_method == FeeSlider.FSMethod.MANUAL:
+            if self._fee_policy.method == FeeMethod.FIXED:
+                self._userFeerate = self.feeRate
+                self.userFeerateChanged.emit()
+            else:
+                self._userFee = self.fee.satsStr
+                self.userFeeChanged.emit()
+
+    def update_inputs_from_tx(self, tx: Transaction):
         inputs = []
         for inp in tx.inputs():
             # addr = self.wallet.adb.get_txin_address(txin)
@@ -277,7 +342,7 @@ class TxFeeSlider(FeeSlider):
             })
         self.inputs = inputs
 
-    def update_outputs_from_tx(self, tx):
+    def update_outputs_from_tx(self, tx: PartialTransaction):
         sm = self._wallet.wallet.lnworker.swap_manager if self._wallet.wallet.lnworker else None
 
         outputs = []
@@ -308,6 +373,14 @@ class TxFeeSlider(FeeSlider):
         else:
             self.warning = ''
 
+    def save_config(self):
+        if self._fee_method == FeeSlider.FSMethod.MANUAL:
+            if self.fee:
+                self.userFee = self.fee.satsStr
+            if self.feeRate:
+                self.userFeerate = self.feeRate
+        super().save_config()
+
 
 class QETxFinalizer(TxFeeSlider):
     _logger = get_logger(__name__)
@@ -315,7 +388,13 @@ class QETxFinalizer(TxFeeSlider):
     finished = pyqtSignal([bool, bool, bool], arguments=['signed', 'saved', 'complete'])
     signError = pyqtSignal([str], arguments=['message'])
 
-    def __init__(self, parent=None, *, make_tx=None, accept=None):
+    def __init__(
+        self,
+        parent=None,
+        *,
+        make_tx: Callable[[int, FeePolicy], PartialTransaction] = None,
+        accept: Callable[[PartialTransaction], None] = None,
+    ):
         super().__init__(parent)
         self.f_make_tx = make_tx
         self.f_accept = accept
@@ -411,6 +490,11 @@ class QETxFinalizer(TxFeeSlider):
             self._valid = False
             self.validChanged.emit()
             return
+        except NoDynamicFeeEstimates:
+            self.warning = _('No dynamic fee estimates available')
+            self._valid = False
+            self.validChanged.emit()
+            return
         except Exception as e:
             self._logger.error(str(e))
             self.warning = repr(e)
@@ -655,12 +739,16 @@ class QETxRbfFeeBumper(TxFeeSlider, TxMonMixin):
             # not initialized yet
             return
 
-        fee_per_kb = self._fee_policy.fee_per_kb(self._wallet.wallet.network)
-        if fee_per_kb is None:
-            # dynamic method and no network
-            self._logger.debug('no fee_per_kb')
-            self.warning = _('Cannot determine dynamic fees, not connected')
-            return
+        if self._fee_policy.method == FeeMethod.FIXED:
+            fee = self._fee_policy.value
+            fee_per_kb = 1000 * fee / self._orig_tx.estimated_size()
+        else:
+            fee_per_kb = self._fee_policy.fee_per_kb(self._wallet.wallet.network)
+            if fee_per_kb is None:
+                # dynamic method and no network
+                self._logger.debug('no fee_per_kb')
+                self.warning = _('Cannot determine dynamic fees, not connected')
+                return
 
         new_fee_rate = fee_per_kb / 1000
         if new_fee_rate <= float(self._oldfee_rate):
@@ -762,12 +850,16 @@ class QETxCanceller(TxFeeSlider, TxMonMixin):
             # not initialized yet
             return
 
-        fee_per_kb = self._fee_policy.fee_per_kb(self._wallet.wallet.network)
-        if fee_per_kb is None:
-            # dynamic method and no network
-            self._logger.debug('no fee_per_kb')
-            self.warning = _('Cannot determine dynamic fees, not connected')
-            return
+        if self._fee_policy.method == FeeMethod.FIXED:
+            fee = self._fee_policy.value
+            fee_per_kb = 1000 * fee / self._orig_tx.estimated_size()
+        else:
+            fee_per_kb = self._fee_policy.fee_per_kb(self._wallet.wallet.network)
+            if fee_per_kb is None:
+                # dynamic method and no network
+                self._logger.debug('no fee_per_kb')
+                self.warning = _('Cannot determine dynamic fees, not connected')
+                return
 
         new_fee_rate = fee_per_kb / 1000
         if new_fee_rate <= float(self._oldfee_rate):
@@ -776,6 +868,13 @@ class QETxCanceller(TxFeeSlider, TxMonMixin):
             self.warning = _("The new fee rate needs to be higher than the old fee rate.")
             return
 
+        if fee_per_kb < self._wallet.wallet.relayfee():
+            self._valid = False
+            self.validChanged.emit()
+            self._logger.warning('feerate too low for relay')
+            self.warning = messages.MSG_RELAYFEE
+            return
+
         try:
             self._tx = self._wallet.wallet.dscancel(
                 tx=self._orig_tx,
@@ -889,11 +988,11 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
         if fee_per_kb is None:
             return None
         package_fee = FeePolicy.estimate_fee_for_feerate(fee_per_kb=fee_per_kb, size=self._total_size)
-        child_fee = package_fee - self._parent_fee
+        return self.get_child_fee_from_total_fee(package_fee)
+
+    def get_child_fee_from_total_fee(self, fee: int) -> int:
+        child_fee = fee - self._parent_fee
         child_fee = min(self._max_fee, child_fee)
-        # pay at least minrelayfee for combined size:
-        min_child_fee = FeePolicy.estimate_fee_for_feerate(fee_per_kb=self._wallet.wallet.relayfee(), size=self._total_size)
-        child_fee = max(min_child_fee, child_fee)
         return child_fee
 
     def tx_verified(self):
@@ -916,19 +1015,21 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
         self.validChanged.emit()
         self.warning = ''
 
-        fee_per_kb = self._fee_policy.fee_per_kb(self._wallet.wallet.network)
-        if fee_per_kb is None:
-            # dynamic method and no network
-            self._logger.debug('no fee_per_kb')
-            self.warning = _('Cannot determine dynamic fees, not connected')
-            return
-
         if self._parent_fee is None:
             self._logger.error(_("Can't CPFP: unknown fee for parent transaction."))
             self.warning = _("Can't CPFP: unknown fee for parent transaction.")
             return
 
-        fee = self.get_child_fee_from_total_feerate(fee_per_kb=fee_per_kb)
+        if self._fee_policy.method == FeeMethod.FIXED:
+            fee = self.get_child_fee_from_total_fee(self._fee_policy.value)
+        else:
+            fee_per_kb = self._fee_policy.fee_per_kb(self._wallet.wallet.network)
+            if fee_per_kb is None:
+                # dynamic method and no network
+                self._logger.debug('no fee_per_kb')
+                self.warning = _('Cannot determine dynamic fees, not connected')
+                return
+            fee = self.get_child_fee_from_total_feerate(fee_per_kb=fee_per_kb)
 
         if fee is None:
             self._logger.warning('no fee')
@@ -938,10 +1039,20 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
             self._logger.warning('max fee exceeded')
             self.warning = _('Max fee exceeded')
             return
+        min_child_fee = FeePolicy.estimate_fee_for_feerate(fee_per_kb=self._wallet.wallet.relayfee(), size=self._total_size)
+        if fee < min_child_fee:
+            self._logger.warning('feerate too low for relay')
+            self.warning = messages.MSG_RELAYFEE
+            return
 
         comb_fee = fee + self._parent_fee
         comb_feerate = comb_fee / self._total_size
 
+        if comb_feerate < (self._parent_fee / self._parent_tx_size):
+            self._logger.debug('combined feerate below parent tx feerate')
+            self.warning = _('Combined feerate should be greater than the parent tx feerate')
+            return
+
         self._fee.satsInt = fee
         self._output_amount.satsInt = self._max_fee - fee
         self.outputAmountChanged.emit()
@@ -961,10 +1072,21 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
 
         self.update_inputs_from_tx(self._new_tx)
         self.update_outputs_from_tx(self._new_tx)
+        self.update_target()
+        self.update_manual_fields()
 
         self._valid = True
         self.validChanged.emit()
 
+    def update_manual_fields(self):
+        if self._fee_method == FeeSlider.FSMethod.MANUAL:
+            if self._fee_policy.method == FeeMethod.FIXED:
+                self._userFeerate = self._total_fee_rate
+                self.userFeerateChanged.emit()
+            else:
+                self._userFee = self._total_fee.satsStr
+                self.userFeeChanged.emit()
+
     @pyqtSlot(result=str)
     def getNewTx(self):
         return str(self._new_tx)
@@ -1004,7 +1126,8 @@ class QETxSweepFinalizer(QETxFinalizer):
         coins, keypairs = copy.deepcopy(self._txins)
         outputs = [PartialTxOutput.from_address_and_value(address, value='!')]
 
-        tx = self._wallet.wallet.make_unsigned_transaction(coins=coins, outputs=outputs, fee=None, rbf=self._rbf, is_sweep=True)
+        tx = self._wallet.wallet.make_unsigned_transaction(
+            coins=coins, outputs=outputs, fee_policy=self._fee_policy, rbf=self._rbf, is_sweep=True)
         self._logger.debug('fee: %d, inputs: %d, outputs: %d' % (tx.get_fee(), len(tx.inputs()), len(tx.outputs())))
 
         tx.sign(keypairs)
@@ -1041,9 +1164,13 @@ class QETxSweepFinalizer(QETxFinalizer):
         try:
             # make unsigned transaction
             tx = self.make_sweep_tx()
-        except Exception as e:
-            self._logger.error(str(e))
-            self.warning = repr(e)
+        except NoDynamicFeeEstimates:
+            self.warning = _('No dynamic fee estimates available')
+            self._valid = False
+            self.validChanged.emit()
+            return
+        except NotEnoughFunds:
+            self.warning = _('Not enough funds')
             self._valid = False
             self.validChanged.emit()
             return
