@@ -78,6 +78,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     otpFailed = pyqtSignal([str, str], arguments=['code', 'message'])
     peersUpdated = pyqtSignal()
     seedRetrieved = pyqtSignal()
+    messageSigned = pyqtSignal([str], arguments=['signature'])
 
     _network_signal = pyqtSignal(str, object)
 
@@ -525,7 +526,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     @pyqtProperty(int, notify=peersUpdated)
     def lightningNumPeers(self):
         if self.isLightning:
-            return self.wallet.lnworker.num_peers()
+            return self.wallet.lnworker.lnpeermgr.num_peers()
         return 0
 
     @pyqtSlot()
@@ -637,14 +638,14 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
 
         # TODO: properly catch server side errors, e.g. bad-txns-inputs-missingorspent
 
-    def save_tx(self, tx: 'PartialTransaction'):
+    def save_tx(self, tx: 'PartialTransaction') -> bool:
         assert tx
 
         try:
             if not self.wallet.adb.add_transaction(tx):
                 self.saveTxError.emit(tx.txid(), 'conflict',
                             _("Transaction could not be saved.") + "\n" + _("It conflicts with current history."))
-                return
+                return False
             self.wallet.save_db()
             self.saveTxSuccess.emit(tx.txid())
             self.historyModel.initModel(True)
@@ -765,6 +766,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
         try:
             self._logger.info('setting new password')
             self.wallet.update_password(current_password, password, encrypt_storage=True)
+            # restore the invariant that all loaded wallets in qml must be unlocked:
             self.wallet.unlock(password)
             return True
         except InvalidPassword as e:
@@ -847,10 +849,12 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     def isAddressMine(self, addr):
         return self.wallet.is_mine(addr)
 
-    @pyqtSlot(str, str, result=str)
+    @pyqtSlot(str, str)
+    @auth_protect(message=_("Sign message?"))
     def signMessage(self, address, message):
         sig = self.wallet.sign_message(address, message, self.password)
-        return base64.b64encode(sig).decode('ascii')
+        result = base64.b64encode(sig).decode('ascii')
+        self.messageSigned.emit(result)
 
     def determine_max(self, *, mktx: Callable[[FeePolicy], PartialTransaction]) -> Tuple[Optional[int], Optional[str]]:
         # TODO: merge with SendTab.spend_max() and move to backend wallet
