@@ -21,6 +21,7 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 import concurrent.futures
+import copy
 from dataclasses import dataclass
 import logging
 import os
@@ -32,6 +33,7 @@ from typing import (
     NamedTuple, Union, TYPE_CHECKING, Tuple, Optional, Callable, Any, Sequence, Dict, Generic, TypeVar, List, Iterable,
     Set, Awaitable
 )
+from types import MappingProxyType
 from datetime import datetime, timezone, timedelta
 import decimal
 from decimal import Decimal
@@ -52,8 +54,10 @@ import functools
 from functools import partial
 from abc import abstractmethod, ABC
 import enum
-from contextlib import nullcontext
+from contextlib import nullcontext, suppress
 import traceback
+import inspect
+import weakref
 
 import aiohttp
 from aiohttp_socks import ProxyConnector, ProxyType
@@ -329,7 +333,7 @@ class MyEncoder(json.JSONEncoder):
         if isinstance(obj, datetime):
             # note: if there is a timezone specified, this will include the offset
             return obj.isoformat(' ', timespec="minutes")
-        if isinstance(obj, set):
+        if isinstance(obj, (set, frozenset)):
             return list(obj)
         if isinstance(obj, bytes): # for nametuples in lnchannel
             return obj.hex()
@@ -350,35 +354,6 @@ class ThreadJob(Logger):
         """Called periodically from the thread"""
         pass
 
-class DebugMem(ThreadJob):
-    '''A handy class for debugging GC memory leaks'''
-    def __init__(self, classes, interval=30):
-        ThreadJob.__init__(self)
-        self.next_time = 0
-        self.classes = classes
-        self.interval = interval
-
-    def mem_stats(self):
-        import gc
-        self.logger.info("Start memscan")
-        gc.collect()
-        objmap = defaultdict(list)
-        for obj in gc.get_objects():
-            for class_ in self.classes:
-                try:
-                    _isinstance = isinstance(obj, class_)
-                except AttributeError:
-                    _isinstance = False
-                if _isinstance:
-                    objmap[class_].append(obj)
-        for class_, objs in objmap.items():
-            self.logger.info(f"{class_.__name__}: {len(objs)}")
-        self.logger.info("Finish memscan")
-
-    def run(self):
-        if time.time() > self.next_time:
-            self.mem_stats()
-            self.next_time = time.time() + self.interval
 
 class DaemonThread(threading.Thread, Logger):
     """ daemon thread that terminates cleanly """
@@ -405,10 +380,14 @@ class DaemonThread(threading.Thread, Logger):
         # malformed or malicious server responses
         with self.job_lock:
             for job in self.jobs:
+                start = time.perf_counter()
                 try:
                     job.run()
                 except Exception as e:
                     self.logger.exception('')
+                duration = time.perf_counter() - start
+                if duration > 0.5:
+                    self.logger.warning(f"thread job {job} blocked {self} DaemonThread for {duration:.2f} s")
 
     def remove_jobs(self, jobs):
         with self.job_lock:
@@ -504,7 +483,7 @@ def profiler(func=None, *, min_threshold: Union[int, float, None] = None):
         if min_threshold is None or t > min_threshold:
             _profiler_logger.debug(f"{func.__qualname__} {t:,.4f} sec")
 
-    if asyncio.iscoroutinefunction(func):
+    if inspect.iscoroutinefunction(func):
         async def do_profile(*args, **kw_args):
             timer_start()
             o = await func(*args, **kw_args)
@@ -889,6 +868,7 @@ UI_UNIT_NAME_FEERATE_SAT_PER_VBYTE = "sat/vbyte"
 UI_UNIT_NAME_FEERATE_SAT_PER_VB = "sat/vB"
 UI_UNIT_NAME_TXSIZE_VBYTES = "vbytes"
 UI_UNIT_NAME_MEMPOOL_MB = "vMB"
+UI_UNIT_NAME_FIXED_SAT = "sat"
 
 
 def format_fee_satoshis(fee, *, num_zeros=0, precision=None):
@@ -1222,7 +1202,7 @@ def is_subpath(long_path: str, short_path: str) -> bool:
 
 def log_exceptions(func):
     """Decorator to log AND re-raise exceptions."""
-    assert asyncio.iscoroutinefunction(func), 'func needs to be a coroutine'
+    assert inspect.iscoroutinefunction(func), 'func needs to be a coroutine'
 
     @functools.wraps(func)
     async def wrapper(*args, **kwargs):
@@ -1243,7 +1223,7 @@ def log_exceptions(func):
 
 def ignore_exceptions(func):
     """Decorator to silently swallow all exceptions."""
-    assert asyncio.iscoroutinefunction(func), 'func needs to be a coroutine'
+    assert inspect.iscoroutinefunction(func), 'func needs to be a coroutine'
 
     @functools.wraps(func)
     async def wrapper(*args, **kwargs):
@@ -1263,26 +1243,37 @@ def with_lock(func):
     return func_wrapper
 
 
-class TxMinedInfo(NamedTuple):
-    height: int                        # height of block that mined tx
+@dataclass(frozen=True, kw_only=True)
+class TxMinedInfo:
+    _height: int                       # height of block that mined tx
     conf: Optional[int] = None         # number of confirmations, SPV verified. >=0, or None (None means unknown)
     timestamp: Optional[int] = None    # timestamp of block that mined tx
     txpos: Optional[int] = None        # position of tx in serialized block
     header_hash: Optional[str] = None  # hash of block that mined tx
     wanted_height: Optional[int] = None  # in case of timelock, min abs block height
 
+    def height(self) -> int:
+        """Treat unverified heights as unconfirmed."""
+        h = self._height
+        if h > 0:
+            if self.conf is not None and self.conf >= 1:
+                return h
+            return 0  # treat it as unconfirmed until SPV-ed
+        else:  # h <= 0
+            return h
+
     def short_id(self) -> Optional[str]:
         if self.txpos is not None and self.txpos >= 0:
-            assert self.height > 0
-            return f"{self.height}x{self.txpos}"
+            assert self.height() > 0
+            return f"{self.height()}x{self.txpos}"
         return None
 
     def is_local_like(self) -> bool:
         """Returns whether the tx is local-like (LOCAL/FUTURE)."""
         from .address_synchronizer import TX_HEIGHT_UNCONFIRMED, TX_HEIGHT_UNCONF_PARENT
-        if self.height > 0:
+        if self.height() > 0:
             return False
-        if self.height in (TX_HEIGHT_UNCONFIRMED, TX_HEIGHT_UNCONF_PARENT):
+        if self.height() in (TX_HEIGHT_UNCONFIRMED, TX_HEIGHT_UNCONF_PARENT):
             return False
         return True
 
@@ -1692,8 +1683,20 @@ def create_and_start_event_loop() -> Tuple[asyncio.AbstractEventLoop,
             loop.run_until_complete(stopping_fut)
         finally:
             # clean-up
+            try:
+                pending_tasks = asyncio.gather(*asyncio.all_tasks(loop), return_exceptions=True)
+                pending_tasks.cancel()
+                with suppress(asyncio.CancelledError):
+                    loop.run_until_complete(pending_tasks)
+                loop.run_until_complete(loop.shutdown_asyncgens())
+                if isinstance(loop, asyncio.BaseEventLoop):
+                    loop.run_until_complete(loop.shutdown_default_executor())
+            except Exception as e:
+                _logger.debug(f"exception when cleaning up asyncio event loop: {e}")
+
             global _asyncio_event_loop
             _asyncio_event_loop = None
+            loop.close()
 
     loop.set_exception_handler(on_exception)
     _set_custom_task_factory(loop)
@@ -1761,7 +1764,7 @@ def run_sync_function_on_asyncio_thread(func: Callable[[], Any], *, block: bool)
 
     For any other thread, we only wait for completion if `block` is True.
     """
-    assert not asyncio.iscoroutinefunction(func), "func must be a non-async function"
+    assert not inspect.iscoroutinefunction(func), "func must be a non-async function"
     asyncio_loop = get_asyncio_loop()
     if get_running_loop() == asyncio_loop:  # we are running on the asyncio thread
         func()
@@ -1851,6 +1854,21 @@ class OrderedDictWithIndex(OrderedDict):
         return ret
 
 
+def make_object_immutable(obj):
+    """Makes the passed object immutable recursively."""
+    allowed_types = (
+        dict, MappingProxyType, list, tuple, set, frozenset, str, int, float, bool, bytes, type(None)
+    )
+    assert isinstance(obj, allowed_types), f"{type(obj)=} cannot be made immutable"
+    if isinstance(obj, (dict, MappingProxyType)):
+        return MappingProxyType({k: make_object_immutable(v) for k, v in obj.items()})
+    elif isinstance(obj, (list, tuple)):
+        return tuple(make_object_immutable(item) for item in obj)
+    elif isinstance(obj, (set, frozenset)):
+        return frozenset(make_object_immutable(item) for item in obj)
+    return obj
+
+
 def multisig_type(wallet_type):
     """If wallet_type is mofn multi-sig, return [m, n],
     otherwise return None."""
@@ -1940,21 +1958,42 @@ class CallbackManager(Logger):
 
     def __init__(self):
         Logger.__init__(self)
-        self.callback_lock = threading.Lock()
-        self.callbacks = defaultdict(list)      # note: needs self.callback_lock
+        self.callback_lock = threading.RLock()
+        self._wcallbacks = defaultdict(set)  # type: Dict[str, Set[weakref.ref[Callable]]]  # note: needs self.callback_lock
 
-    def register_callback(self, func, events):
+    @staticmethod
+    def _wcb_from_any_callback(cb: Callable) -> weakref.ref[Callable]:
+        assert callable(cb), type(cb)
+        if isinstance(cb, weakref.ref):  # no-op
+            return cb
+        elif inspect.ismethod(cb):  # instance method, such as for a subclass of EventListener
+            return WeakMethodProper(cb)
+        else:  # proper function? e.g. used by lnpeer unit tests
+            return weakref.ref(cb)
+
+    def register_callback(self, cb: Callable, events: Sequence[str]) -> None:
+        wcb = self._wcb_from_any_callback(cb)
         with self.callback_lock:
             for event in events:
-                self.callbacks[event].append(func)
+                self._wcallbacks[event].add(wcb)
 
-    def unregister_callback(self, callback):
+    def unregister_callback(self, cb: Callable) -> None:
+        wcb = self._wcb_from_any_callback(cb)
         with self.callback_lock:
-            for callbacks in self.callbacks.values():
-                if callback in callbacks:
-                    callbacks.remove(callback)
+            # note: ^ callback_lock needs to be re-entrant, as we can now trigger __del__, which also takes the lock
+            for callbacks in self._wcallbacks.values():
+                if wcb in callbacks:
+                    callbacks.remove(wcb)
 
-    def trigger_callback(self, event, *args):
+    def count_all_callbacks(self) -> int:
+        with self.callback_lock:
+            return sum(len(cbs) for cbs in self._wcallbacks.values())
+
+    def clear_all_callbacks(self) -> None:
+        with self.callback_lock:
+            self._wcallbacks.clear()
+
+    def trigger_callback(self, event: str, *args) -> None:
         """Trigger a callback with given arguments.
         Can be called from any thread. The callback itself will get scheduled
         on the event loop.
@@ -1962,9 +2001,12 @@ class CallbackManager(Logger):
         loop = get_asyncio_loop()
         assert loop.is_running(), "event loop not running"
         with self.callback_lock:
-            callbacks = self.callbacks[event][:]
-        for callback in callbacks:
-            if asyncio.iscoroutinefunction(callback):  # async cb
+            wcallbacks = copy.copy(self._wcallbacks[event])
+        for wcb in wcallbacks:
+            callback = wcb()
+            if callback is None:
+                continue
+            if inspect.iscoroutinefunction(callback):  # async cb
                 fut = asyncio.run_coroutine_threadsafe(callback(*args), loop)
 
                 def on_done(fut_: concurrent.futures.Future):
@@ -1985,11 +2027,28 @@ unregister_callback = callback_mgr.unregister_callback
 _event_listeners = defaultdict(set)  # type: Dict[str, Set[str]]
 
 
+class WeakMethodProper(weakref.WeakMethod):
+    """Unlike weakref.WeakMethod, this class has an __eq__ I can trust."""
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        meth = self()
+        self._my_id = (id(meth.__self__), id(meth.__func__))
+
+    def __hash__(self):
+        return hash(self._my_id)
+
+    def __eq__(self, other):
+        if not isinstance(other, WeakMethodProper):
+            return False
+        return self._my_id == other._my_id
+
+
 class EventListener:
     """Use as a mixin for a class that has methods to be triggered on events.
     - Methods that receive the callbacks should be named "on_event_*" and decorated with @event_listener.
-    - register_callbacks() should be called exactly once per instance of EventListener, e.g. in __init__
+    - register_callbacks() should be called once per instance of EventListener, e.g. in __init__
     - unregister_callbacks() should be called at least once, e.g. when the instance is destroyed
+        - as fallback, __del__() also calls unregister_callbacks()
     """
 
     def _list_callbacks(self):
@@ -2011,6 +2070,9 @@ class EventListener:
             #_logger.debug(f'unregistering callback {method}')
             unregister_callback(method)
 
+    def __del__(self):
+        self.unregister_callbacks()
+
 
 def event_listener(func):
     """To be used in subclasses of EventListener only. (how to enforce this programmatically?)"""
@@ -2359,7 +2421,7 @@ class OnchainHistoryItem(NamedTuple):
             'txid': self.txid,
             'amount_sat': self.amount_sat,
             'fee_sat': self.fee_sat,
-            'height': self.tx_mined_status.height,
+            'height': self.tx_mined_status.height(),
             'confirmations': self.tx_mined_status.conf,
             'timestamp': self.tx_mined_status.timestamp,
             'monotonic_timestamp': self.monotonic_timestamp,
